@@ -32,6 +32,7 @@ from ..gateways.disk.test import file_path_is_writable
 from ..models.dist import Dist
 from ..models.match_spec import MatchSpec
 from ..models.records import PackageCacheRecord, PackageRecord, PackageRef
+from ..utils import human_bytes
 
 try:
     from cytoolz.itertoolz import concat, concatv, groupby
@@ -610,7 +611,12 @@ class ProgressiveFetchExtract(object):
         if cache_axn is None and extract_axn is None:
             return
 
-        desc = "%s %s" % (prec_or_spec.name, prec_or_spec.version)
+        desc = "%s-%s" % (prec_or_spec.name, prec_or_spec.version)
+        if len(desc) > 20:
+            desc = desc[:20]
+        size = getattr(prec_or_spec, 'size', None)
+        desc = "%-20s | %7s | " % (desc, size and human_bytes(size) or '')
+
         progress_bar = ProgressBar(desc, not context.verbosity and not context.quiet, context.json)
 
         download_total = 0.75  # fraction of progress for download; the rest goes to extract
